/*
 * Copyright 2009 Cristian Vasile Mocanu (cvmocanu@gmail.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.mocanu.intellij.cic;

import com.intellij.openapi.util.TextRange;
import com.intellij.psi.PsiComment;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiReference;
import com.intellij.psi.PsiReferenceProvider;
import com.intellij.util.ProcessingContext;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ClassesInCommentsPsiReferenceProvider extends PsiReferenceProvider {
//    private static final Logger LOGGER = Logger.getInstance(ClassesInCommentsPsiReferenceProvider.class.getName());

    private static final Pattern CLASS_NAME_PATTERN = Pattern.compile("(?:\\b(?:[_a-zA-Z0-9])+\\b\\.)*\\b(?:[_a-zA-Z0-9])+\\b");

    @NotNull
    @Override
    public PsiReference[] getReferencesByElement(@NotNull PsiElement element, @NotNull ProcessingContext context) {
        String elementText = element.getText();
        if (elementText == null) {
            return PsiReference.EMPTY_ARRAY;
        }

        PsiComment comment = (PsiComment) element;

        boolean softReference = true;

        List<ReferenceToClassesFromComments> references = new ArrayList<ReferenceToClassesFromComments>();
        Matcher matcher = CLASS_NAME_PATTERN.matcher(elementText);
        while (matcher.find()) {
            String potentialClassName = matcher.group();

            references.add(
                    new ReferenceToClassesFromComments(
                            comment,
                            new TextRange(matcher.start(), matcher.end()),
                            softReference,
                            potentialClassName
                    )
            );
        }

        return references.toArray(new PsiReference[references.size()]);
    }
}
